#* Introduction to Model Specification

#* Specifying the Measurement Model

influence_mm <- constructs(
  # specifications of your various constructs go here
)

# Note that in the above code, we are capturing our definition of all our
# constructs in a variable called `influence_mm` for later use. Within the
# `constructs()` function, we must define each of our constructs.

# Constructs can be either *common factors* or *composites*, and SEMinR allows
# your models to have one or both types of constructs in your models if your
# estimation method permits it.

#* Common-Factor Models

# Common factors are latent, unmeasurable concepts whose variance is reflected
# in base measurement items, which in turn can be thought of as symptoms or
# results of the underlying factor. Common factors are specified in SEMinR using
# the `reflective()` function:



# Reflective function example:
influence_mm <- constructs(
  reflective("PL", multi_items("pl_", 1:4)),
  reflective("SIC", multi_items("sic_", 1:7)),
  reflective("PI", multi_items("pi_", 1:5)))

# Note the that the `reflective()` function requires two parameters: -
# `construct_name`: the name of the construct, such as `"PL"` above -
# `item_names`: the names of the items for that construct

# The item names could be named separately and combined using the `c()`
# function of R as follow:

reflective("PL", c("pl_1", "pl_2", "pl_3", "pl_4"))

# However, typing the items separately can feel repetitive and present
# opportunities for typos. Thus, SEMinR offers the `multi_items()` function that
# can capture multiple repetitive item names as a simple sequence:

reflective("PL", multi_items("pl_", 1:4))

#* Composite Models

# In contrast, composites derive their meaning from their measurement items,
# and so are modeled as merely a weighted average of their items. Composites are
# specified in SEMinR using the `composite()` function. We can redefine the
# earlier constructs as composites:

influence_mm <- constructs(
  composite("PL", multi_items("pl_", 1:4)),
  composite("SIC", multi_items("sic_", 1:7)),
  composite("PI", multi_items("pi_", 1:5)))

# Note the that the `composite()` function can take the following parameters: -
# `construct_name`: the name of the construct, such as `"PL"` above -
# `item_names`: the names of the items for that construct - `weights`:
# optionally specify how items of a construct should be weighted in taking their
# average (defaults to `correlation_weights`)

# Composites have scores that are computed by taking a weighted average of
# their items. There are two possible weighting schemes that SEMinR supports:

#** Correlation weighting

# The default weighting scheme for items of composites is to use their
# individual correlations with the overall composite as their weight:

composite("PL", multi_items("pl_", 1:4), weights = correlation_weights)

#** Regression weighting

# Alternatively, items of a composite can be regressed over their composite,
# and their regression coefficients can be used as their weights:

composite("PL", multi_items("pl_", 1:4), weights = regression_weights)

#* Structural Models

# The relationships between constructs are defined within the `relationships()`
# function, using the `paths()` function for groups of causal paths.

influence_sm <- relationships(
  paths(from = "SIC", to = c("PL", "PI")),
  paths(from = c("SIC", "PL"), to = "PI"))

# The above set of relationships is saved in the `influence_sm` object for
# later estimation.

# The individual causal paths are specified in the `paths()` functions above.
# Note that groups of paths can be specified together if they share the same
# antecedent or outcome construct. For example, the following code defines two
# causal paths, both from the `SIC` construct with one going to the `PL`
# construct and one to the `PI` construct.

  paths(from = "SIC", to = c("PL", "PI"))

#* Estimation Methods

# There are two dominant sets of methods for estimating and evaluating
# constructs or entire SEMs: covariance-based estimation or partial least
# squares modeling.

#** Covariance-based Estimation

# Covariance-based methods are preferred when all constructs are common
# factors. SEMinR supports evaluating the measurement model of common factors
# using confirmatory factor analysis (CFA), and the structural model using
# covariance-based structural equation modeling (CBSEM)

#*** Confirmatory Factor Analysis

# A common validation of the measurement of common factor models is conducting
# a confirmatory factor analysis (CFA). SEMinR can use the `Lavaan` package to
# estimate parameters of a measurement model using CFA:

influence_mm <- constructs(
  reflective("PL", multi_items("pl_", 1:4)),
  reflective("SIC", multi_items("sic_", 1:7)),
  reflective("PI", multi_items("pi_", 1:5)))

influencer_data <- read.csv("data/influencer_data_book_final.csv")
cfa_model <- estimate_cfa(data = influencer_data, influence_mm)

cat(cfa_model$lavaan_model)

cfa_report <- summary(cfa_model)

# The summary contains many useful results that researchers will want to
# analyze and report:

cfa_report$loadings



# Quality Metrics:
cfa_report$quality$fit
cfa_report$quality$reliability

# Correlations:
cfa_report$descriptives$correlations$constructs
round( cfa_report$descriptives$correlations$items, 2)

#*** Common-Factor Structural Models

# With a common-factor measurement model in place, you may define and estimate
# a structural model. The preferred method of estimating common factor SEMs is
# using covariance-based SEM — we one again leverage the powerful `Lavaan`
# package to estimate for CBSEM.

influence_sm <- relationships(
  paths(from = c("PL", "SIC"), to = "PI"))

cbsem_model <- estimate_cbsem(influencer_data, influence_mm, influence_sm)

plot(cbsem_model$construct_scores[, c("PL", "PI")])

# However, a full report of results comes again from the `summary()`:

cbsem_report <- summary(cbsem_model)

# This summary object will now contain all essential metrics, including fit
# indices that researchers wish to report. There are many fit metrics reported
# so here we will show only a subset of 10 criteria from all the fit metrics:

cbsem_report$quality$fit$all[1:10]

cbsem_report$quality$fit$curated

#** Partial Least Squares

# Partial Least Squares Path Modeling (PLS-PM) is often the preferred method to
# estimate composite models. PLS-PM does not support estimating measurement
# models alone; rather, the measurement model and structural model are estimated
# together.

influence_mm <- constructs(
  composite("PL", multi_items("pl_", 1:4)),
  composite("SIC", multi_items("sic_", 1:7)),
  composite("PI", multi_items("pi_", 1:5)))

influence_sm <- relationships(
  paths(from = c("PL", "SIC"), to = "PI"))

pls_model <- estimate_pls(influencer_data, influence_mm, influence_sm)

plot(pls_model$construct_scores[, c("PL", "PI")])

# A full report with more extensive is once again available using the
# `summary()` function:

summary(pls_model)

plot(pls_model)

#* Reusability and Composability

# The syntax for SEMinR allows researchers to create reusable and composable
# blocks of code. For example, one can use the same measurement model for
# multiple structural models, such as a proposed model and an alternative model.

influence_mm <- constructs(
  reflective("PL", multi_items("pl_", 1:4)),
  reflective("SIC", multi_items("sic_", 1:7)),
  reflective("PQ", multi_items("pq_", 1:4)),
  reflective("PIC", multi_items("pic_", 1:5)),
  reflective("PI", multi_items("pi_", 1:5)),
  reflective("WTP", single_item("wtp")))

proposed_sm <- relationships(
  paths(from = c("PL", "SIC"), to = c("PQ", "PIC")),
  paths(from = c("PQ", "PIC"), to = c("PI", "WTP")))

alternative_sm <- relationships(
  paths(from = c("PL", "SIC"), to = c("PI", "WTP")))

proposed_model <- estimate_cbsem(influencer_data, influence_mm, proposed_sm)
alternative_model <- estimate_cbsem(influencer_data, influence_mm, alternative_sm)

# As you get more comfortable with R, you will find even more ways to make your
# model syntax elegant and readable. The following example shows how to create
# named groups of constructs to make the structural models highly readable:



# Assuming measurement model and constructs are already defined...
antecedents <- c("PL", "SIC")
mediators   <- c("PQ", "PIC")
outcomes    <- c("PI", "WTP")

proposed_sm <- relationships(
  paths(from = antecedents, to = mediators),
  paths(from = mediators, to = outcomes))

alternative_sm <- relationships(
  paths(from = antecedents, to = outcomes))

proposed_model <- estimate_cbsem(influencer_data, influence_mm, proposed_sm)
alternative_model <- estimate_cbsem(influencer_data, influence_mm, alternative_sm)

# There are many more options and outputs you can avail in SEMinR, but I hope
# this gives a gentle introduction to major topics. Please see our Advanced
# Topics booklet for more details.

#* Join us

# We recommend you join our community of users and developers on various social
# media and collaborative platforms.

#** Collaboration and Feedback

# We have a Github repository where you can see our latest instructions:

# https://github.com/sem-in-r/seminr

# Please use this repository to report suspected bugs that you wish us to track
# and fix. You will have to create a Github account to report such issues:

# https://github.com/sem-in-r/seminr/issues

#** Social Media

# We have a Facebook group of like-minded researchers where we ask questions
# about SEMinR and also keep abreast of developments around SEM.

# https://www.facebook.com/groups/seminr/

